@@ -4,6 +4,8 @@ import (
 	"context"
 	"fmt"
 	"log"
+	"os"
+	"strconv"
 	"strings"
 	"sync"
 	"time"
@@ -82,7 +84,6 @@ func (state *refreshConnectionState) close() {
 // and update the database schema and search path to reflect the required connections
 // return whether any changes have been made
 func (state *refreshConnectionState) refreshConnections(ctx context.Context) {
-
 	// if there was an error (other than a connection error, which will NOT have been assigned to res),
 	// set state of all incomplete connections to error
 	defer func() {
@@ -101,7 +102,7 @@ func (state *refreshConnectionState) refreshConnections(ctx context.Context) {
 		return
 	}
 
-	log.Printf("[INFO] created connection updates")
+	log.Printf("[INFO] created connectionUpdates")
 
 	// delete the connection state file - it will be rewritten when we are complete
 	log.Printf("[INFO] deleting connections state file")
@@ -357,9 +358,18 @@ func (state *refreshConnectionState) executeUpdateSetsInParallel(ctx context.Con
 	var wg sync.WaitGroup
 	var errChan = make(chan *connectionError)
 
-	// use as many goroutines as we have connections
-	var maxUpdateThreads = int64(state.pool.Config().MaxConns)
-	sem := semaphore.NewWeighted(maxUpdateThreads)
+	// default to running a single update at a time
+	var maxParallel = int64(1)
+	// allow override of this behaviour vis env var
+	if envMaxStr, ok := os.LookupEnv("STEAMPIPE_UPDATE_SCHEMA_MAX_PARALLEL"); ok {
+		envMax, err := strconv.Atoi(envMaxStr)
+		if err == nil {
+			maxParallel = int64(envMax)
+		}
+	}
+	log.Printf("[INFO] executeUpdateSetsInParallel - maxParallel= %d", maxParallel)
+
+	sem := semaphore.NewWeighted(maxParallel)
 
 	go func() {
 		for {
@@ -378,6 +388,13 @@ func (state *refreshConnectionState) executeUpdateSetsInParallel(ctx context.Con
 		}
 	}()
 
+	// allow disabling of schema clone via env var
+	var cloneSchemaEnabled = true
+	if envClone, ok := os.LookupEnv("STEAMPIPE_CLONE_SCHEMA"); ok {
+		cloneSchemaEnabled = strings.ToLower(envClone) == "true"
+	}
+	log.Printf("[INFO] executeUpdateForConnections - cloneSchema=%v", cloneSchemaEnabled)
+
 	// each update may be multiple connections, to execute in order
 	for _, states := range updates {
 		wg.Add(1)
@@ -393,7 +410,7 @@ func (state *refreshConnectionState) executeUpdateSetsInParallel(ctx context.Con
 				sem.Release(1)
 			}()
 
-			state.executeUpdateForConnections(ctx, errChan, connectionStates...)
+			state.executeUpdateForConnections(ctx, errChan, cloneSchemaEnabled, connectionStates...)
 		}(states)
 
 	}
@@ -405,7 +422,8 @@ func (state *refreshConnectionState) executeUpdateSetsInParallel(ctx context.Con
 }
 
 // syncronously execute the update queries for one or more connections
-func (state *refreshConnectionState) executeUpdateForConnections(ctx context.Context, errChan chan *connectionError, connectionStates ...*steampipeconfig.ConnectionState) {
+func (state *refreshConnectionState) executeUpdateForConnections(ctx context.Context, errChan chan *connectionError, cloneSchemaEnabled bool, connectionStates ...*steampipeconfig.ConnectionState) {
+
 	for _, connectionState := range connectionStates {
 		connectionName := connectionState.ConnectionName
 		remoteSchema := utils.PluginFQNToSchemaName(connectionState.Plugin)
@@ -414,7 +432,7 @@ func (state *refreshConnectionState) executeUpdateForConnections(ctx context.Con
 		state.exemplarSchemaMapMut.Lock()
 		// is this plugin in the exemplarSchemaMap
 		exemplarSchemaName, haveExemplarSchema := state.exemplarSchemaMap[connectionState.Plugin]
-		if haveExemplarSchema {
+		if haveExemplarSchema && cloneSchemaEnabled {
 			// we can clone!
 			sql = getCloneSchemaQuery(sql, exemplarSchemaName, connectionState)
 		} else {
