@@ -12,7 +12,7 @@ then
 fi
 
 # set this to the source file for development
-export BATS_PATH=$MY_PATH/lib/bats/bin/bats
+export BATS_PATH=$MY_PATH/lib/bats-core/bin/bats
 export LIB_BATS_ASSERT=$MY_PATH/lib/bats-assert
 export LIB_BATS_SUPPORT=$MY_PATH/lib/bats-support
 export TEST_DATA_DIR=$MY_PATH/test_data/templates
@@ -30,7 +30,7 @@ export CONTROL_RENDERING_TEST_MOD=$MY_PATH/test_data/control_rendering_test_mod
 export STEAMPIPE_CONNECTION_WATCHER=false
 export STEAMPIPE_INTROSPECTION=info
 export DEFAULT_WORKSPACE_PROFILE_LOCATION=$MY_PATH/test_data/source_files/workspace_profile_default
-
+export STEAMPIPE_SYNC_REFRESH=true
 # from GH action env variables
 export SPIPETOOLS_PG_CONN_STRING=$SPIPETOOLS_PG_CONN_STRING
 export SPIPETOOLS_TOKEN=$SPIPETOOLS_TOKEN
@@ -53,13 +53,15 @@ echo " ___) | || (_| | |  | |_| | | | | (_| |   | |  __/\__ \ |_\__ \\"
 echo "|____/ \__\__,_|_|   \__|_|_| |_|\__, |   |_|\___||___/\__|___/"
 echo "                                 |___/                         "
 
-export PATH=$MY_PATH/lib/bats/bin:$PATH
+export PATH=$MY_PATH/lib/bats-core/bin:$PATH
 
 if [[ ! ${STEAMPIPE_INSTALL_DIR} ]];
 then
   export STEAMPIPE_INSTALL_DIR="$HOME/.steampipe"
 fi
 
+batversion=$(bats --version)
+echo $batversion
 echo "Running with STEAMPIPE_INSTALL_DIR set to $STEAMPIPE_INSTALL_DIR"
 
 if [ $# -eq 0 ]; then
