@@ -1,10 +1,12 @@
+import { ReactNode } from "react";
+
 const componentsMap = {};
 
 const getComponent = (key: string) => componentsMap[key];
 
 const registerComponent = (
   key: string,
-  component: (props: any) => JSX.Element | null
+  component: (props: any) => ReactNode
 ) => {
   componentsMap[key] = component;
 };
